// This class keeps track of the state of a list of things that can be
// drawn on a canvas.  The state can be either:
//    CLEAN = everything that needs drawing was drawn
//    APPEND = some new items need to be drawn
//    REDRAW = the entire list needs to be redrawn

import java.util.*;

public class DrawList<E> extends LinkedList<E> {
    private final int CLEAN = 0;
    private final int APPEND = 1;
    private final int REDRAW = 2;
    private int lastdrawn = 0;
    private boolean redraw = true;
    private boolean dispatched = false;
    private Vector<DLListener<E>> listeners = new Vector<DLListener<E>>();

    public DrawList() {
	super();
    }

    public void push(E item) {
	this.add(item);
	dispatched = false;
    }

    public void clear() {
	super.clear();
	setRedraw();
    }

    public boolean hasUndrawn() {
	boolean hasUn = (this.size() != lastdrawn);
	if (!dispatched) {
	    dispatch();
	}
	return hasUn;
    }

    public E getUndrawn() {
	redraw = false;
	//	System.out.println("Returning undrawn item "+lastdrawn);
	lastdrawn = lastdrawn + 1;
	return this.get(lastdrawn - 1);
    }

    public void setRedraw() {
	lastdrawn = 0;
	redraw = true;
	dispatched = false;
	dispatch();
    }

    public boolean needRedraw() {
	return redraw;
    }

    public int getState () {
	if (redraw) {
	    return REDRAW;
	} else {
	    if (hasUndrawn()) {
		return APPEND;
	    } else {
		return CLEAN;
	    }
	}
    }

    public void addListener(DLListener<E> L)
    {
	listeners.add(L);
    }

    public void dispatch()
    {
	dispatched = true;
	Iterator<DLListener<E>> i = listeners.iterator();
	DLEvent<E> e = new DLEvent<E>(this);
	while (i.hasNext()) {
	    i.next().DLUpdated(e);
	}
    }
}
