import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.awt.geom.*;
import java.math.*;
import java.util.*;
import java.awt.font.*;

public class HistCanvas extends Canvas {
    int PADDING = 5;
    int WIDTH,HEIGHT;
    float SCALEFACTOR;
    float CX, CY;
    float PIXEL;
    float SCALEDHEIGHT;
    float MAX, YMAX;
    AffineTransform T, TINV;
    int bins;

    Image buffer;
    Image fixedbg;
    Graphics gbuffer;

    Histogrammer H;
    Boolean blank = true;
   
    ProbDist target;

    public HistCanvas(int width, int height, double max, double ymax, int bins, ProbDist target) {
	WIDTH = width;
	HEIGHT = height;
	MAX = (float)max;
	YMAX = (float)ymax;
	this.bins = bins;
	this.target = target;
	setSize(WIDTH,HEIGHT);
	setPreferredSize(new Dimension(WIDTH,HEIGHT));
	
	// Compute transformation
	SCALEFACTOR = (WIDTH - 2.0f*PADDING) / MAX;
	
	PIXEL = (1.0f / SCALEFACTOR);
	SCALEDHEIGHT = ((HEIGHT - 2.0f*PADDING) / SCALEFACTOR);
	T = new AffineTransform();
	T.translate(PADDING,HEIGHT-PADDING);
	T.scale(SCALEFACTOR,-1.0f*SCALEFACTOR);
	try {
	    TINV = T.createInverse();
	}
	catch(Exception e) {
	    // Very bad... scale + translate was not invertible?
	    // In that case make TINV the identity.
	    TINV = new AffineTransform();
	}
	clear();
    }
    
    public void addNotify() {
	super.addNotify();
	// Create fixed background
	fixedbg = createImage(WIDTH,HEIGHT);
	paintFixed((Graphics2D)fixedbg.getGraphics());

	// Create buffer
	buffer = createImage(WIDTH,HEIGHT);
	gbuffer = buffer.getGraphics();
    }

    public void paintFixed(Graphics2D g) {
	// Pretty rendering
	g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
	// Clear
	g.setColor(Color.black);
	g.fillRect(0,0,getWidth(),getHeight());

	// From now on, use complex coordinates
	AffineTransform saved = g.getTransform();
	g.transform(T);

	if (target != null) {
	    g.setColor(Color.blue);
	    g.setStroke(new BasicStroke(2.0f*PIXEL));
	    GeneralPath P = new GeneralPath();
	    P.moveTo(0.0f,(float)target.density(0.0) * SCALEDHEIGHT / YMAX);
	    for (int i=1;i<20;i++) {
		double t = MAX * (double)i / 20.0;
		P.lineTo((float)(t + 0.5*PIXEL),(float)target.density(t) * SCALEDHEIGHT / YMAX);
	    }
	    g.draw(P);
	}

	g.setColor(Color.gray);
	g.setStroke(new BasicStroke(1.0f*PIXEL));
	GeneralPath P = new GeneralPath();
	P.moveTo(0.0f,-0.5f * PADDING * PIXEL);
	P.lineTo(0.0f,SCALEDHEIGHT);
	P.moveTo(-0.5f * PADDING * PIXEL,0.0f);
	P.lineTo(MAX + 0.5f * PADDING * PIXEL,0.0f);
	g.draw(P);
	
	// Revert to device coordinates
	g.setTransform(saved);
    }
    
    public void paint(Graphics g) {
        paintReal(gbuffer);
        g.drawImage(buffer, 0, 0, this);
    }
    
    public void paintReal(Graphics g2) {
	g2.drawImage(fixedbg, 0, 0, this);
	Graphics2D g = (Graphics2D)g2;
	g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
	// From now on, use complex coordinates
	AffineTransform saved = g.getTransform();
	g.transform(T);	
	
	g.setColor(new Color(255,80,80,130));
	g.setStroke(new BasicStroke(1.0f*PIXEL));
	if (!blank) {
	    for (int i=0; i<bins; i++) {
		g.fill(makeBar(H.getBinMin(i),H.getBinMax(i),H.getDensity(i)*SCALEDHEIGHT / YMAX));
	    }
	}
	
	// Revert to device coordinates
	g.setTransform(saved);
    }
    
    private Shape makePoint(double x, double y, double r)
    {
	return(new Ellipse2D.Double(x-r*PIXEL,
				    y-r*PIXEL,
				    2.0*r*PIXEL,2.0*r*PIXEL));
    }
    
    private Shape makeBar(double xmin, double xmax, double h)
    {
	return(new Rectangle2D.Double(xmin+1.0*PIXEL,0.0,xmax-xmin-0.5*PIXEL,h));
    }

    public void clear() {
	H = new Histogrammer(0.0,MAX,bins);
	blank = true;
    }

    public void add(double x) {
	H.add(x);
	blank = false;
    }
}
